import asyncio
from asyncio import Future
from typing import Dict, List

from web_foundation.app.app import App
from web_foundation.kernel.isolates.channel import IChannel
from web_foundation.kernel.isolates.pipes import IMessage
from loguru import logger


class IDistributor:
    channels: Dict[str, IChannel]

    def __init__(self, app: App):
        self.channels = {}
        self.channels.update({iso.name: iso.channel for iso in app.isolates})

    async def on_channel_sent(self, msg: IMessage):
        logger.warning(msg)
        await self.channels.get(msg.destination).consume_pipe.write(msg)

    def preform(self) -> List[Future]:
        tasks = []
        for channel in self.channels.values():
            tasks.append(asyncio.ensure_future(
                channel.listen_produce(
                    self.on_channel_sent
                )))
        return tasks
