import asyncio
from asyncio import Future
from typing import List

from aioprocessing import AioProcess

from web_foundation.kernel.isolates.channel import IChannel


class Isolate:
    name: str
    debug: bool
    _named_isolates: List[str] = []
    _channel: IChannel
    _proc: AioProcess
    _async_mode: bool

    def __init__(self, name: str, async_mode: bool = False, debug: bool = False):
        if name in self._named_isolates:
            raise AttributeError("Isolate.name nums be uniq")
        self.name = name
        self._named_isolates.append(name)
        self._channel = IChannel(self.name)
        self._async_mode = async_mode
        self._proc = AioProcess(target=self._startup)
        self.debug = debug

    async def async_run(self):
        raise NotImplementedError

    def sync_run(self):
        raise NotImplementedError

    def _startup(self):
        if self._async_mode:
            asyncio.run(self.async_run())
        else:
            self.sync_run()

    async def _exec(self):
        self._proc.start()

    def perform(self) -> Future:
        return asyncio.ensure_future(self._exec())

    @property
    def channel(self) -> IChannel:
        return self._channel

    @property
    def pid(self) -> int:
        return self._proc.pid

    @property
    def process(self) -> AioProcess:
        return self._proc
